<?php

namespace WilokePricing\Controllers;

use Elementor\Controls_Manager;
use Elementor\Utils;
use Elementor\Widget_Base;
use Timber\Timber;

class PricingAddon extends Widget_Base
{
	public static $aSettings               = [];
	public        $aHighlightItemsSettings = [];
	public        $aBasicItemsSettings     = [];

	public function get_name()
	{
		return esc_html__("Wiloke Pricing Table", "wiloke-pricing");
	}

	public function get_stack($with_common_controls = true)
	{
		return parent::get_stack(false);
	}

	public function get_title()
	{
		return esc_html__("Wiloke Pricing", "wiloke-pricing");
	}

	public function get_script_depends()
	{
		return ['wiloke-pricing-script'];
	}

	public function get_icon()
	{
		return 'eicon-price-table';
	}

	public function get_style_depends()
	{
		return ['wiloke-pricing'];
	}

	public function get_categories()
	{
		return ['basic'];
	}

	public function get_keywords()
	{
		return ['pricing', 'wiloke', "table", "pricingtable", "price", "price table", "list"];
	}

	private function getHighlightItemSettings()
	{
		if ($this->aHighlightItemsSettings) {
			return $this->aHighlightItemsSettings;
		}

		$this->aHighlightItemsSettings['backgroundColor'] = self::$aSettings['highlight_backgroundColor'];
		$this->aHighlightItemsSettings['color'] = self::$aSettings['highlight_color'];
		$this->aHighlightItemsSettings['colorPrimary'] = self::$aSettings['highlight_colorPrimary'];
		$this->aHighlightItemsSettings['colorSecondary'] = self::$aSettings['highlight_colorSecondary'];

		return $this->aHighlightItemsSettings;
	}

	private function getBasicItemSettings()
	{
		if ($this->aBasicItemsSettings) {
			return $this->aBasicItemsSettings;
		}

		$this->aBasicItemsSettings['colorPrimary'] = self::$aSettings['basic_colorPrimary'];
		$this->aBasicItemsSettings['colorSecondary'] = self::$aSettings['basic_colorSecondary'];

		return $this->aBasicItemsSettings;
	}

	private function parseItems($aSettings)
	{
		$aItems = [];
		foreach ($aSettings['data'] as $aItem) {
			$buttonText = $aItem['button_text'];
			$buttonLink = $aItem['button_link'];
			$aItem['button'] = [
				'text' => $buttonText,
				'url'  => $buttonLink
			];

			if ($aItem['topIcon']) {
				$aItem['icon'] = $aItem['topIcon']['value'];
			}

			unset($aItem['topIcon']);
			unset($aItem['button_text']);
			unset($aItem['button_link']);

			if ($aItem['highlight']) {
				$aItem['styles'] = $this->getHighlightItemSettings();
			} else {
				$aItem['styles'] = $this->getBasicItemSettings();
			}

			$aItems[] = $aItem;
		}

		return $aItems;
	}

	protected function register_controls()
	{
		$this->start_controls_section(
			'wiloke_basic_setting',
			[
				'label' => esc_html__('Settings', 'wiloke-pricing'),
				'tab'   => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'data',
			[
				'label'   => esc_html__('Items', 'wiloke-pricing'),
				'type'    => Controls_Manager::REPEATER,
				'fields'  => [
					[
						'name'    => 'highlight',
						'label'   => esc_html__('Is Highlight?', 'wiloke-pricing'),
						'type'    => Controls_Manager::SWITCHER,
						'default' => false
					],
					[
						'name'    => 'title',
						'label'   => esc_html__('Title', 'wiloke-pricing'),
						'type'    => Controls_Manager::TEXT,
						'default' => 'Basic'
					],
					[
						'name'    => 'subtitle',
						'label'   => esc_html__('Sub Title', 'wiloke-pricing'),
						'type'    => Controls_Manager::TEXT,
						'default' => 'For most business that want to use this app.'
					],
					[
						'name'    => 'topIcon',
						'label'   => esc_html__('Icon', 'wiloke-pricing'),
						'type'    => Controls_Manager::ICONS,
						'default' => [
							'value'   => 'fa fa-money',
							'library' => 'fa-regular'
						]
					],
					[
						'name'    => 'price',
						'label'   => esc_html__('Price', 'wiloke-pricing'),
						'type'    => Controls_Manager::TEXT,
						'default' => 'Free'
					],
					[
						'name'    => 'period',
						'label'   => esc_html__('Period', 'wiloke-pricing'),
						'type'    => Controls_Manager::TEXT,
						'default' => 'Month'
					],
					[
						'name'    => 'features',
						'label'   => esc_html__('Features', 'wiloke-pricing'),
						'type'    => Controls_Manager::WYSIWYG,
						'default' => '<ul>
				   <li>Feature 1</li>
				   <li>Feature 2</li>
				   <li><del>Feature 3</del></li>
				    </ul>'
					],
					[
						'name'    => 'button_text',
						'label'   => esc_html__('Button Name', 'wiloke-pricing'),
						'type'    => Controls_Manager::TEXT,
						'default' => 'Get Started'
					],
					[
						'name'    => 'button_link',
						'label'   => esc_html__('Button Link', 'wiloke-pricing'),
						'type'    => Controls_Manager::TEXT,
						'default' => '#'
					],
				],
				'default' => [
					[
						'title'       => 'Basic',
						'subtitle'    => 'For most business that want to use this app.',
						'price'       => 'Free',
						'period'      => 'Month',
						'button_link' => '#',
						'button_text' => 'Get Started',
						'features'    => '<ul>
				   <li>Feature 1</li>
				   <li>Feature 2</li>
				   <li><del>Feature 3</del></li>
				    </ul>'
					],
				]
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'wiloke_style_settings',
			[
				'label' => esc_html__('Style', 'wiloke-pricing'),
				'tab'   => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->start_controls_tabs(
			'style_tabs'
		);

		$this->start_controls_tab(
			'general_settings_tab',
			[
				'label' => esc_html__('Style', 'wiloke-pricing'),
			]
		);
		$this->add_control(
			'variant',
			[
				'label'       => esc_html__('Style', 'wiloke-pricing'),
				'description' => esc_html__('If you want to use Style 2, please fill out the Section with a non-white background color',
					'wiloke-pricing'),
				'type'        => Controls_Manager::SELECT,
				'options'     => [
					'pricingSection1' => esc_html__('Style 1', 'wiloke-pricing'),
					'pricingSection2' => esc_html__('Style 2', 'wiloke-pricing'),
					'pricingSection3' => esc_html__('Style 3', 'wiloke-pricing'),
					'pricingSection4' => esc_html__('Style 4', 'wiloke-pricing')
				],
				'default'     => 'pricingSection1'
			]
		);
		$this->add_control(
			'response',
			[
				'label'      => esc_html__('Items per row', 'wiloke-pricing'),
				'type'       => Controls_Manager::SELECT,
				'options'    => [
					1 => esc_html__('1 item / row', 'wiloke-pricing'),
					2 => esc_html__('2 items / row', 'wiloke-pricing'),
					3 => esc_html__('3 items / row', 'wiloke-pricing'),
					4 => esc_html__('4 items / row', 'wiloke-pricing'),
					5 => esc_html__('5 items / row', 'wiloke-pricing'),
					6 => esc_html__('6 items / row', 'wiloke-pricing'),
				],
				'default'    => 3,
				'conditions' => [
					'variant' => [
						[
							'name'     => 'variant',
							'operator' => '!==',
							'value'    => 'pricingSection4'
						]
					]
				]
			]
		);
		$this->end_controls_tab();

		$this->start_controls_tab(
			'highlight_style_tab',
			[
				'label' => esc_html__('Highlight Table Color', 'wiloke-pricing'),
			]
		);
		$this->add_control(
			'highlight_backgroundColor',
			[
				'label'   => esc_html__('Background Color', 'wiloke-pricing'),
				'type'    => Controls_Manager::COLOR,
				'default' => '#5f2fe1'
			]
		);
		$this->add_control(
			'highlight_color',
			[
				'label'   => esc_html__('Text Color', 'wiloke-pricing'),
				'type'    => Controls_Manager::COLOR,
				'default' => '#fff'
			]
		);
		$this->add_control(
			'highlight_colorPrimary',
			[
				'label'   => esc_html__('Primary Color', 'wiloke-pricing'),
				'type'    => Controls_Manager::COLOR,
				'default' => '#fbd3c7'
			]
		);
		$this->add_control(
			'highlight_colorSecondary',
			[
				'label'   => esc_html__('Secondary Color', 'wiloke-pricing'),
				'type'    => Controls_Manager::COLOR,
				'default' => '#5f2fe1'
			]
		);
		$this->end_controls_tab();


		$this->start_controls_tab(
			'basic_style_tab',
			[
				'label' => esc_html__('Basic Tables Color', 'wiloke-pricing'),
			]
		);
		$this->add_control(
			'basic_colorPrimary',
			[
				'label'   => esc_html__('Primary Color', 'wiloke-pricing'),
				'type'    => Controls_Manager::COLOR,
				'default' => 'transparent'
			]
		);
		$this->add_control(
			'basic_colorSecondary',
			[
				'label'   => esc_html__('Secondary Color', 'wiloke-pricing'),
				'type'    => Controls_Manager::COLOR,
				'default' => '#5f2fe1'
			]
		);
		$this->end_controls_tab();

		$this->end_controls_tabs();
		$this->end_controls_section();
	}

	protected function render()
	{
		self::$aSettings = $this->get_settings_for_display();
		Timber::render(plugin_dir_path(__FILE__) . "../Views/pricingSection.twig", [
			"data"    => $this->parseItems(self::$aSettings),
			"variant" => self::$aSettings["variant"]
		]);
	}
}
