(function () {
    "use strict";

    function elementorCheckedDecorator(Constructor, selector, target, ...rest) {
        class WithElementorChecked extends Constructor {
            constructor() {
                super();
                this.intervalId = -1;
                this.checkElementor();
            }

            checkElementor() {
                if (typeof elementorFrontend !== "undefined") {
                    this.intervalId = setInterval(() => {
                        this.els = document.querySelectorAll(selector);
                        const targetEls = document.querySelectorAll(target);
                        if (!targetEls.length && this.els.length) {
                            this.init();
                            // clearInterval(this.intervalId);
                        }
                    }, 3000);
                }
            }
        }

        return new WithElementorChecked(...rest);
    }

    class PricingList {
        constructor() {
            this.els = document.querySelectorAll(".wil-pricing-list");
            this.observer = null;
            this.init();
            this.update();
        }

        test() {
            document.querySelectorAll(".wil-pricing-col").forEach((el) => {
                el.addEventListener("click", () => {
                    el.parentNode.insertAdjacentHTML("beforeend", el.outerHTML);
                });
            });
        }

        handleElement(el) {
            const listEls = el.querySelectorAll("li");
            listEls.forEach((listEl) => {
                const delEl = listEl.children[0];
                if (delEl) {
                    listEl.classList.add("wil-pricing-list__del");
                } else {
                    listEl.classList.add("wil-pricing-list__ins");
                }
            });
        }

        update() {
            this.observer = new MutationObserver(() => {
                this.init();
            });
            if (this.observer) {
                this.observer.disconnect();
            }
            this.els.forEach((el) => {
                this.observer.observe(el, {
                    childList: true,
                    subtree: true,
                });
            });
        }

        init() {
            if (this.els.length) {
                this.els.forEach(this.handleElement);
            }
        }
    }

    elementorCheckedDecorator(PricingList, ".wil-pricing-list", ".wil-pricing-list__ins", );

    class Pricing4 {
        constructor() {
            this.state = {
                indexActive: 0,
            };
            this.rowEls = document.querySelectorAll(".wil-pricing-row--4");
            /** @type {HTMLElement[] | null} */
            this.pricingEls = null;
            this.init();
        }

        /**
         * @private
         */
        handlePricingEls() {
            const {indexActive} = this.state;
            if (this.pricingEls) {
                this.pricingEls.forEach((pricingEl) => {
                    pricingEl.style.display = "none";
                });
                this.pricingEls[indexActive].style.display = "block";
            }
        }

        /**
         * @private
         * @param {MouseEvent} event
         */
        handlePricingSmallClick(event) {
            const {indexActive} = this.state;
            const pricingSmallEl = event.currentTarget;
            const pricingSmallEls = pricingSmallEl.parentNode.querySelectorAll(
                ".wil-pricing-small4"
            );
            const index = Array.prototype.indexOf.call(
                pricingSmallEls,
                pricingSmallEl
            );
            if (index !== indexActive) {
                pricingSmallEls.forEach((pricingSmallEl) => {
                    pricingSmallEl.classList.remove("wil-pricing-small4--active");
                });
                pricingSmallEl.classList.add("wil-pricing-small4--active");
            }
            this.state.indexActive = index;
            this.handlePricingEls();
        }

        /**
         * @private
         * @param {HTMLElement} pricingSmallEl
         * @param {number} index
         */
        handlePricingSmallEl(pricingSmallEl, index) {
            const {indexActive} = this.state;
            if (index === indexActive) {
                pricingSmallEl.classList.add("wil-pricing-small4--active");
            }
            pricingSmallEl.addEventListener(
                "click",
                this.handlePricingSmallClick.bind(this)
            );
        }

        /**
         * @private
         * @param {HTMLElement} rowEl
         */
        handleRow(rowEl) {
            const colLeftEl = rowEl.querySelector(".wil-pricing-col--left");
            const colRightEl = rowEl.querySelector(".wil-pricing-col--right");
            this.pricingEls = colRightEl.querySelectorAll(".wil-pricing4");
            const pricingSmallEls = colLeftEl.querySelectorAll(".wil-pricing-small4");
            pricingSmallEls.forEach(this.handlePricingSmallEl.bind(this));
            this.handlePricingEls();
        }

        init() {
            this.rowEls.forEach(this.handleRow.bind(this));
        }
    }

    elementorCheckedDecorator(Pricing4, ".wil-pricing-list", ".wil-pricing-list__ins");
})();
